@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * CPIface graphic spectrum analysers
  *
@@ -529,11 +529,13 @@ static void plDrawStripes (struct cpifaceSessionAPI_t *cpifaceSession)
 	}
 }
 
-static void strSetMode (struct cpifaceSessionAPI_t *cpifaceSession)
+static int strSetMode (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	cpiSetGraphMode(plStripeBig);
+	if ((cpiSetGraphMode(plStripeBig)) < 0)
+		return -1;
 	plPrepareStripes();
 	plPrepareStripeScr (cpifaceSession);
+	return 0;
 }
 
 static int plStripeKey (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t key)
