@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2011-'20 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2011-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * CPIface text track/pattern display
  *
@@ -369,7 +369,9 @@ static void preparepatgen (struct cpifaceSessionAPI_t *cpifaceSession, int pat,
 		}
 		lastprow=curlen;
 		if ((firstrow+curlen-firstprow)>=plPatBufH)
+		{
 			lastprow=plPatBufH-firstrow-firstprow-1;
+		}
 
 		for (i=firstprow; i<lastprow; i++)
 		{
@@ -421,8 +423,10 @@ static void preparepatgen (struct cpifaceSessionAPI_t *cpifaceSession, int pat,
 		firstprow=0;
 		firstpat++;
 
-		if (firstrow>=plPatBufH)
+		if (firstrow >= (plPatBufH-1))
+		{
 			break;
+		}
 	}
 	for (i=firstrow; i<plPatBufH; i++)
 		writestring(plPatBuf[i], 0, COLBACK, "", CONSOLE_MAX_X);
@@ -893,8 +897,7 @@ void cpiTrkSetup2 (struct cpifaceSessionAPI_t *cpifaceSession, const struct cpit
 	cpiTextRegisterMode (cpifaceSession, &cpiTModeTrack);
 }
 
-void __attribute__ ((visibility ("internal"))) cpiTrackInit (void)
+OCP_INTERNAL void cpiTrackInit (void)
 {
 	plTrackActive=cfGetProfileBool2(cfScreenSec, "screen", "pattern", 1, 1);
 }
-
