@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2020-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2020-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Code to decode TAR file archives
  *
@@ -132,8 +132,6 @@ static int tar_file_filesize_ready (struct ocpfile_t *);
 static void tar_filehandle_ref (struct ocpfilehandle_t *);
 static void tar_filehandle_unref (struct ocpfilehandle_t *);
 static int tar_filehandle_seek_set (struct ocpfilehandle_t *, int64_t pos);
-static int tar_filehandle_seek_cur (struct ocpfilehandle_t *, int64_t pos);
-static int tar_filehandle_seek_end (struct ocpfilehandle_t *, int64_t pos);
 static uint64_t tar_filehandle_getpos (struct ocpfilehandle_t *);
 static int tar_filehandle_eof (struct ocpfilehandle_t *);
 static int tar_filehandle_error (struct ocpfilehandle_t *);
@@ -164,7 +162,7 @@ static uint32_t tar_instance_add (struct tar_instance_t *self,
                                   const uint64_t  fileoffset);
 
 /* in the blob, we will switch / into \0 temporary as we parse them */
-static void tar_instance_decode_blob (struct tar_instance_t *self, uint8_t *blob, size_t blobsize)
+static void tar_instance_decode_blob (struct tar_instance_t *self, uint8_t *blob, uint32_t blobsize)
 {
 
 	uint8_t *eos;
@@ -223,7 +221,7 @@ static void tar_instance_decode_blob (struct tar_instance_t *self, uint8_t *blob
 	tar_translate_complete (self);
 }
 
-static void tar_instance_encode_blob (struct tar_instance_t *self, uint8_t **blob, size_t *blobfill)
+static void tar_instance_encode_blob (struct tar_instance_t *self, uint8_t **blob, uint32_t *blobfill)
 {
 	uint32_t counter;
 	uint32_t blobsize = 0;
@@ -373,7 +371,8 @@ static uint32_t tar_instance_add_create_dir (struct tar_instance_t *self,
 	                dirdb_ref,
 	                0, /* refcount */
 	                1, /* is_archive */
-	                0  /* is_playlist */);
+	                0, /* is_playlist */
+	                self->archive_file->compression);
 
 	self->dirs[self->dir_fill]->owner = self;
 	self->dirs[self->dir_fill]->dir_parent = dir_parent;
@@ -404,7 +403,7 @@ static uint32_t tar_instance_add_file (struct tar_instance_t *self,
 	uint32_t *prev, iter;
 	uint32_t dirdb_ref;
 
-	DEBUG_PRINT ("[TAR] add_file: %s %s\n", Filepath, Filename);
+	DEBUG_PRINT ("[TAR] add_file: %s %s (offset=%llu size=%llu)\n", Filepath, Filename, (long long unsigned)fileoffset, (long long unsigned)filesize);
 
 	if (self->file_fill == self->file_size)
 	{
@@ -446,7 +445,8 @@ static uint32_t tar_instance_add_file (struct tar_instance_t *self,
 	                 0, /* filename_override */
 	                 dirdb_ref,
 	                 0, /* refcount */
-	                 0  /* is_nodetect */);
+	                 0, /* is_nodetect */
+	                 COMPRESSION_ADD_STORE (self->archive_file->compression));
 
 	self->files[self->file_fill]->owner      = self;
 	self->files[self->file_fill]->dir_parent = dir_parent;
@@ -572,7 +572,8 @@ struct ocpdir_t *tar_check (const struct ocpdirdecompressor_t *self, struct ocpf
 	                file->dirdb_ref,
 	                0, /* refcount */
 	                1, /* is_archive */
-	                0  /* is_playlist */);
+	                0, /* is_playlist */
+	                file->compression);
 
 	file->parent->ref (file->parent);
 	iter->dirs[0]->owner = iter;
@@ -594,7 +595,7 @@ struct ocpdir_t *tar_check (const struct ocpdirdecompressor_t *self, struct ocpf
 	{
 		const char *filename = 0;
 		uint8_t *metadata = 0;
-		size_t metadatasize = 0;
+		uint32_t metadatasize = 0;
 
 		dirdbGetName_internalstr (iter->archive_file->dirdb_ref, &filename);
 		if (!adbMetaGet (filename, iter->archive_file->filesize (iter->archive_file), "TAR", &metadata, &metadatasize))
@@ -885,7 +886,7 @@ static int tar_dir_readdir_iterate (ocpdirhandle_pt _self)
 		{
 			const char *filename;
 			uint8_t *metadata = 0;
-			size_t metadatasize = 0;
+			uint32_t metadatasize = 0;
 
 			self->dir->owner->ready = 1;
 			// tar_translate_complete (iter); /* in theory, two instances might scan at the same time, so we only clean-up then in the destructor
@@ -1056,6 +1057,7 @@ static int tar_filehandle_read (struct ocpfilehandle_t *_self, void *dst, int le
 		self->error = 1;
 		return 0;
 	}
+	DEBUG_PRINT ("TAR seek_set %llu\n", (long long unsigned)(self->file->fileoffset + self->filepos));
 	if (filehandle->seek_set (filehandle, self->file->fileoffset + self->filepos) < 0)
 	{
 		self->error = 1;
@@ -1104,8 +1106,6 @@ static struct ocpfilehandle_t *tar_file_open (struct ocpfile_t *_self)
 	                       tar_filehandle_unref,
 	                       _self,
 	                       tar_filehandle_seek_set,
-	                       tar_filehandle_seek_cur,
-	                       tar_filehandle_seek_end,
 	                       tar_filehandle_getpos,
 	                       tar_filehandle_eof,
 	                       tar_filehandle_error,
@@ -1114,14 +1114,14 @@ static struct ocpfilehandle_t *tar_file_open (struct ocpfile_t *_self)
 	                       tar_filehandle_filesize,
 	                       tar_filehandle_filesize_ready,
 	                       0, /* filename_override */
-	                       dirdbRef (self->head.dirdb_ref, dirdb_use_filehandle));
+	                       dirdbRef (self->head.dirdb_ref, dirdb_use_filehandle),
+	                       1 /* refcount */
+	);
 
 	retval->file = self;
 
-	DEBUG_PRINT ("We just created a TAR handle, REF it\n");
-	retval->head.ref (&retval->head);
-	DEBUG_PRINT ("\n");
-
+	DEBUG_PRINT ("We just created a TAR handle, REF the source: \"%s\"\n", self->orig_full_filepath);
+	tar_instance_ref (self->owner);
 	tar_io_ref (self->owner);
 
 	return &retval->head;
@@ -1143,12 +1143,7 @@ static void tar_filehandle_ref (struct ocpfilehandle_t *_self)
 	struct tar_instance_filehandle_t *self = (struct tar_instance_filehandle_t *)_self;
 
 	DEBUG_PRINT ("tar_filehandle_ref (old count = %d)\n", self->head.refcount);
-	if (!self->head.refcount)
-	{
-		tar_instance_ref (self->file->owner);
-	}
 	self->head.refcount++;
-	DEBUG_PRINT ("\n");
 }
 
 static void tar_filehandle_unref (struct ocpfilehandle_t *_self)
@@ -1160,7 +1155,6 @@ static void tar_filehandle_unref (struct ocpfilehandle_t *_self)
 	self->head.refcount--;
 	if (!self->head.refcount)
 	{
-		dirdbUnref (self->head.dirdb_ref, dirdb_use_filehandle);
 		tar_io_unref (self->file->owner);
 		tar_instance_unref (self->file->owner);
 		free (self);
@@ -1182,45 +1176,6 @@ static int tar_filehandle_seek_set (struct ocpfilehandle_t *_self, int64_t pos)
 	return 0;
 }
 
-static int tar_filehandle_seek_cur (struct ocpfilehandle_t *_self, int64_t pos)
-{
-	struct tar_instance_filehandle_t *self = (struct tar_instance_filehandle_t *)_self;
-
-	if (pos <= 0)
-	{
-		if (pos == INT64_MIN) return -1; /* we never have files this size */
-		if ((-pos) > self->filepos) return -1;
-		self->filepos += pos;
-	} else {
-		/* check for overflow */
-		if ((int64_t)(pos + self->filepos) < 0) return -1;
-
-		if ((pos + self->filepos) > self->file->filesize) return -1;
-		self->filepos += pos;
-	}
-
-	self->error = 0;
-
-	return 0;
-}
-
-static int tar_filehandle_seek_end (struct ocpfilehandle_t *_self, int64_t pos)
-{
-	struct tar_instance_filehandle_t *self = (struct tar_instance_filehandle_t *)_self;
-
-	if (pos > 0) return -1;
-
-	if (pos == INT64_MIN) return -1; /* we never have files this size */
-
-	if (pos < -(int64_t)(self->file->filesize)) return -1;
-
-	self->filepos = self->file->filesize + pos;
-
-	self->error = 0;
-
-	return 0;
-}
-
 static uint64_t tar_filehandle_getpos (struct ocpfilehandle_t *_self)
 {
 	struct tar_instance_filehandle_t *self = (struct tar_instance_filehandle_t *)_self;
@@ -1407,7 +1362,7 @@ static void tar_set_byuser_string (struct ocpdir_t *_self, const char *byuser)
 	/* update adbMeta */
 	{
 		uint8_t *metadata = 0;
-		size_t metadatasize = 0;
+		uint32_t metadatasize = 0;
 		const char *filename = 0;
 
 		tar_instance_encode_blob (self->owner, &metadata, &metadatasize);
