@@ -9,9 +9,61 @@ struct ocpdirdecompressor_t;
 
 typedef void *ocpdirhandle_pt;
 
+#include "filesystem-filehandle-cache.h"
+
 #define FILESIZE_STREAM  UINT64_C(0xffffffffffffffff) /* STREAM - so we recommend to open and analyze the file asap */
 #define FILESIZE_ERROR   UINT64_C(0xfffffffffffffffe)
 
+#define COMPRESSION_NONE   0 /* available directly on the file-system */
+#define COMPRESSION_STORE  1 /* .tar, .zip(STORE)                     */
+#define COMPRESSION_STREAM 2 /* .gz .bzip2 .zip                       */
+#define COMPRESSION_SOLID  3 /* solid .tar.gz .tar.bz2                */
+#define COMPRESSION_SOLID1 4 /* .gz.tar.gz ....                       */
+#define COMPRESSION_SOLID2 5 /* ... */
+#define COMPRESSION_SOLID3 6 /* ... */
+#define COMPRESSION_SOLID4 7 /* ... */
+#define COMPRESSION_REMOTE 127
+
+static inline uint8_t COMPRESSION_ADD_STORE (uint8_t parent)
+{
+	uint8_t retval = (parent >= COMPRESSION_STREAM) ? parent+1 : parent | 1;
+	if (parent == COMPRESSION_REMOTE)
+	{
+		return COMPRESSION_REMOTE;
+	}
+	if (retval > COMPRESSION_SOLID4)
+	{
+		retval = COMPRESSION_SOLID4;
+	}
+	return retval;
+}
+static inline uint8_t COMPRESSION_ADD_STREAM (uint8_t parent)
+{
+	uint8_t retval =  parent + 2;
+	if (parent == COMPRESSION_REMOTE)
+	{
+		return COMPRESSION_REMOTE;
+	}
+	if (retval > COMPRESSION_SOLID4)
+	{
+		retval = COMPRESSION_SOLID4;
+	}
+	return retval;
+}
+static inline uint8_t COMPRESSION_ADD_SOLID (uint8_t parent)
+{
+	uint8_t retval =  parent + 3;
+	if (parent == COMPRESSION_REMOTE)
+	{
+		return COMPRESSION_REMOTE;
+	}
+	if (retval > COMPRESSION_SOLID4)
+	{
+		retval = COMPRESSION_SOLID4;
+	}
+	return retval;
+}
+
 struct ocpdir_charset_override_API_t
 {
 	void (*get_default_string)(struct ocpdir_t *self, const char **label, const char **charset);
@@ -43,6 +95,7 @@ struct ocpdir_t /* can be an archive */
 	int refcount; /* internal use by all object variants */
 	uint8_t is_archive;
 	uint8_t is_playlist;
+	uint8_t compression;
 };
 
 struct ocpdir_t  *ocpdir_t_fill_default_readdir_dir  (struct ocpdir_t *, uint32_t dirdb_ref);
@@ -65,7 +118,8 @@ static inline void ocpdir_t_fill (
 	int dirdb_ref,
 	int refcount,
 	uint8_t is_archive,
-	uint8_t is_playlist)
+	uint8_t is_playlist,
+	uint8_t compression)
 {
 	s->ref                  = ref;
 	s->unref                = unref;
@@ -81,6 +135,7 @@ static inline void ocpdir_t_fill (
 	s->refcount             = refcount;
 	s->is_archive           = is_archive;
 	s->is_playlist          = is_playlist;
+	s->compression          = compression;
 }
 
 #ifdef _DIRDB_H
@@ -125,6 +180,9 @@ struct ocpfile_t /* can be virtual */
 	void (*unref)(struct ocpfile_t *);
 	struct ocpdir_t *parent;
 	struct ocpfilehandle_t *(*open)(struct ocpfile_t *);
+#ifndef FILEHANDLE_CACHE_DISABLE
+	struct ocpfilehandle_t *(*real_open)(struct ocpfile_t *);
+#endif
 
 	uint64_t (*filesize)(struct ocpfile_t *); // can return FILESIZE_STREAM FILESIZE_ERROR
 	int (*filesize_ready)(struct ocpfile_t *); // if this is false, asking for filesize might be very slow
@@ -133,10 +191,15 @@ struct ocpfile_t /* can be virtual */
 	int dirdb_ref;
 	int refcount; /* internal use by all object variants */
 	uint8_t is_nodetect; /* do not use mdbReadInfo on this file please */
+	uint8_t compression;
 };
 
 const char *ocpfile_t_fill_default_filename_override (struct ocpfile_t *);
 
+#ifndef FILEHANDLE_CACHE_DISABLE
+static struct ocpfilehandle_t *ocpfilehandle_cache_open_wrap (struct ocpfile_t *f);
+#endif
+
 static inline void ocpfile_t_fill (
 	struct ocpfile_t *s,
 	void (*ref)(struct ocpfile_t *),
@@ -148,18 +211,25 @@ static inline void ocpfile_t_fill (
 	const char *(*filename_override) (struct ocpfile_t *),
 	int dirdb_ref,
 	int refcount,
-	uint8_t is_nodetect)
+	uint8_t is_nodetect,
+	uint8_t compression)
 {
 	s->ref            = ref;
 	s->unref          = unref;
 	s->parent         = parent;
+#ifndef FILEHANDLE_CACHE_DISABLE
+	s->open           = ocpfilehandle_cache_open_wrap;
+	s->real_open      = open;
+#else
 	s->open           = open;
+#endif
 	s->filesize       = filesize;
 	s->filesize_ready = filesize_ready;
 	s->filename_override = filename_override ? filename_override : ocpfile_t_fill_default_filename_override;
 	s->dirdb_ref      = dirdb_ref;
 	s->refcount       = refcount;
 	s->is_nodetect    = is_nodetect;
+	s->compression    = compression;
 }
 
 struct ocpfilehandle_t
@@ -170,8 +240,6 @@ struct ocpfilehandle_t
 	struct ocpfile_t *origin;
 
 	int (*seek_set)(struct ocpfilehandle_t *, int64_t pos); /* returns 0 for OK, and -1 on error, should use positive numbers */
-	int (*seek_cur)(struct ocpfilehandle_t *, int64_t pos); /* returns 0 for OK, and -1 on error */
-	int (*seek_end)(struct ocpfilehandle_t *, int64_t pos); /* returns 0 for OK, and -1 on error, should use negative numbers */
 	uint64_t (*getpos)(struct ocpfilehandle_t *);
 
 	int (*eof)(struct ocpfilehandle_t *);   /* 0 = more data, 1 = EOF, -1 = error - probably tried to read beyond EOF */
@@ -189,6 +257,25 @@ struct ocpfilehandle_t
 	int refcount; /* internal use by all object variants */
 };
 
+#ifndef FILEHANDLE_CACHE_DISABLE
+static struct ocpfilehandle_t *ocpfilehandle_cache_open_wrap (struct ocpfile_t *f)
+{
+	struct ocpfilehandle_t *retval1, *retval2;
+	retval1 = f->real_open (f);
+	if (!retval1)
+	{
+		return 0;
+	}
+	retval2 = cache_filehandle_open (retval1);
+	if (!retval2)
+	{
+		return retval1;
+	}
+	retval1->unref (retval1);
+	return retval2;
+}
+#endif
+
 int ocpfilehandle_t_fill_default_ioctl (struct ocpfilehandle_t *, const char *cmd, void *ptr);
 const char *ocpfilehandle_t_fill_default_filename_override (struct ocpfilehandle_t *);
 
@@ -198,8 +285,6 @@ static inline void ocpfilehandle_t_fill (
 	void (*unref)(struct ocpfilehandle_t *),
 	struct ocpfile_t *origin,
 	int (*seek_set)(struct ocpfilehandle_t *, int64_t pos),
-	int (*seek_cur)(struct ocpfilehandle_t *, int64_t pos),
-	int (*seek_end)(struct ocpfilehandle_t *, int64_t pos),
 	uint64_t (*getpos)(struct ocpfilehandle_t *),
 	int (*eof)(struct ocpfilehandle_t *),
 	int (*error)(struct ocpfilehandle_t *),
@@ -208,14 +293,13 @@ static inline void ocpfilehandle_t_fill (
 	uint64_t (*filesize)(struct ocpfilehandle_t *),
 	int (*filesize_ready)(struct ocpfilehandle_t *),
 	const char *(*filename_override) (struct ocpfilehandle_t *),
-	int dirdb_ref)
+	int dirdb_ref,
+	int refcount)
 {
 	s->ref               = ref;
 	s->unref             = unref;
 	s->origin            = origin;
 	s->seek_set          = seek_set;
-	s->seek_cur          = seek_cur;
-	s->seek_end          = seek_end;
 	s->getpos            = getpos;
 	s->eof               = eof;
 	s->error             = error;
@@ -225,6 +309,7 @@ static inline void ocpfilehandle_t_fill (
 	s->filesize_ready    = filesize_ready;
 	s->filename_override = filename_override ? filename_override : ocpfilehandle_t_fill_default_filename_override;
 	s->dirdb_ref         = dirdb_ref;
+	s->refcount          = refcount;
 }
 
 static inline int ocpfilehandle_read_uint8     (struct ocpfilehandle_t *s, uint8_t *dst) /* returns 0 for OK, and -1 on error */
