@@ -16,6 +16,10 @@ struct ocpfilehandle_t;
 
 #define MODULETYPE(str) ((uint32_t)(((uint8_t)str[0]) | (((uint8_t)(str[0]?str[1]:0))<<8) | (((uint8_t)((str[0]&&str[1])?str[2]:0))<<16) | (((uint8_t)((str[0]&&str[1]&&str[2])?str[3]:0))<<24)) )
 
+#define mtUnRead 0
+#define mtUnknownStr "UNKN"
+#define mtUnknown    MODULETYPE(mtUnknownStr)
+
 struct __attribute__((packed)) moduletype
 {
 	union
@@ -75,7 +79,7 @@ int mdbGetModuleType (uint32_t fileref, struct moduletype *dst);
 int mdbInfoIsAvailable (uint32_t fileref); // used to be mdbInfoRead
 int mdbReadInfo(struct moduleinfostruct *m, struct ocpfilehandle_t *f);
 int mdbWriteModuleInfo(uint32_t fileref, struct moduleinfostruct *m); // returns zero on error
-void mdbScan(struct ocpfile_t *file, uint32_t mdb_ref);
+void mdbScan(struct ocpfile_t *file, uint32_t mdb_ref, struct ocpfilehandle_t **retain); // if retain is non-zero, do not unref filehandle, but pass it to caller
 int mdbInit (const struct configAPI_t *configAPI); // returns zero on error
 void mdbUpdate(void);
 void mdbClose(void);
