@@ -65,6 +65,10 @@ static void modland_com_mirror_Draw (
 	int mlHeight = 12 + NUM_MIRRORS;
 	int mlWidth = 74;
 
+#if (CONSOLE_MIN_Y < 20)
+# error modland_com_mirror_Draw() requires CONSOLE_MIN_Y >= 20
+#endif
+
 	int mlTop = (plScrHeight - mlHeight) / 2;
 	int mlLeft = (plScrWidth - mlWidth) / 2;
 	int i;
