@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * the file-object code used by the File selector ][
  *
@@ -60,6 +60,7 @@ void modlist_free (struct modlist *modlist)
 			modlist->files[i].file->unref (modlist->files[i].file);
 			modlist->files[i].file = 0;
 		}
+		free (modlist->files[i].utf8_casefolded);
 	}
 	free (modlist->files);
 	free (modlist->sortindex);
@@ -84,7 +85,7 @@ struct modlistentry *modlist_get (const struct modlist *modlist, unsigned int in
 	return &modlist->files[modlist->sortindex[index]];
 }
 
-void modlist_append (struct modlist *modlist, struct modlistentry *entry)
+static void _modlist_append (struct modlist *modlist, struct modlistentry *entry) /* steals entry->utf8_casefolded */
 {
 	if (!entry)
 		return;
@@ -125,6 +126,14 @@ void modlist_append (struct modlist *modlist, struct modlistentry *entry)
 	modlist->num++;
 }
 
+void modlist_append (struct modlist *modlist, struct modlistentry *entry)
+{
+	if (!entry)
+		return;
+	_modlist_append (modlist, entry);
+	modlist->files[modlist->num - 1].utf8_casefolded = strdup (modlist->files[modlist->num - 1].utf8_casefolded);
+}
+
 void modlist_append_dir (struct modlist *modlist, struct ocpdir_t *dir)
 {
 	struct modlistentry entry = {{0}};
@@ -140,9 +149,11 @@ void modlist_append_dir (struct modlist *modlist, struct ocpdir_t *dir)
 	utf8_XdotY_name (8, 3, entry.utf8_8_dot_3, childpath);
 	utf8_XdotY_name (16, 3, entry.utf8_16_dot_3, childpath);
 
+	entry.utf8_casefolded = utf8_casefold (childpath);
+
 	entry.mdb_ref = UINT32_MAX;
 
-	modlist_append (modlist, &entry);
+	_modlist_append (modlist, &entry);
 }
 
 void modlist_append_dotdot (struct modlist *modlist, struct ocpdir_t *dir)
@@ -159,9 +170,11 @@ void modlist_append_dotdot (struct modlist *modlist, struct ocpdir_t *dir)
 	strcpy (entry.utf8_8_dot_3,  "..");
 	strcpy (entry.utf8_16_dot_3, "..");
 
+	entry.utf8_casefolded = strdup ("..");
+
 	entry.mdb_ref = UINT32_MAX;
 
-	modlist_append (modlist, &entry);
+	_modlist_append (modlist, &entry);
 }
 
 void modlist_append_drive (struct modlist *modlist, struct dmDrive *drive)
@@ -180,12 +193,14 @@ void modlist_append_drive (struct modlist *modlist, struct dmDrive *drive)
 	utf8_XdotY_name (8, 3, entry.utf8_8_dot_3, childpath);
 	utf8_XdotY_name (16, 3, entry.utf8_16_dot_3, childpath);
 
+	entry.utf8_casefolded = utf8_casefold (childpath);
+
 	entry.mdb_ref = UINT32_MAX;
 
-	modlist_append (modlist, &entry);
+	_modlist_append (modlist, &entry);
 }
 
-void modlist_append_file (struct modlist *modlist, struct ocpfile_t *file, int ismod)
+void modlist_append_file (struct modlist *modlist, struct ocpfile_t *file, int ismod, int prescanhint, struct ocpfilehandle_t **retain)
 {
 	struct modlistentry entry = {{0}};
 	const char *childpath = 0;
@@ -205,6 +220,8 @@ void modlist_append_file (struct modlist *modlist, struct ocpfile_t *file, int i
 	utf8_XdotY_name (8, 3, entry.utf8_8_dot_3, childpath);
 	utf8_XdotY_name (16, 3, entry.utf8_16_dot_3, childpath);
 
+	entry.utf8_casefolded = utf8_casefold (childpath);
+
 	if (ismod)
 	{
 		entry.mdb_ref = mdbGetModuleReference2 (file->dirdb_ref, file->filesize (file));
@@ -212,7 +229,16 @@ void modlist_append_file (struct modlist *modlist, struct ocpfile_t *file, int i
 		entry.mdb_ref = UINT32_MAX;
 	}
 
-	modlist_append (modlist, &entry);
+	if (entry.mdb_ref != UINT32_MAX)
+	{
+		if (prescanhint && (!mdbInfoIsAvailable(entry.mdb_ref)))
+		{
+			mdbScan (file, entry.mdb_ref, retain);
+			entry.flags |= MODLIST_FLAG_SCANNED;
+		}
+	}
+
+	_modlist_append (modlist, &entry);
 }
 
 
@@ -250,6 +276,8 @@ void modlist_clear(struct modlist *modlist)
 			modlist->files[i].file->unref (modlist->files[i].file);
 			modlist->files[i].file = 0;
 		}
+		free (modlist->files[i].utf8_casefolded);
+		modlist->files[i].utf8_casefolded = 0;
 	}
 	modlist->num = 0;
 }
@@ -438,12 +466,62 @@ static int mlecmp (const void *a, const void *b)
 
 	if (e1->flags & MODLIST_FLAG_DRV)
 	{
-		return strcasecmp (e1->utf8_16_dot_3, e2->utf8_16_dot_3);
+		return strcmp (e1->utf8_16_dot_3, e2->utf8_16_dot_3); /* On windows, drive letters are all capital letters, and should appear infront of the other protocols, hence we use strcmp */
 	}
 
+	/* First compare them as casefolded */
+	{
+		int retval = strcmp (e1->utf8_casefolded, e2->utf8_casefolded);
+		if (retval != 0)
+		{
+			return retval;
+		}
+	}
+
+	/* If they match after casefolded, seperate base regular strcmp, so we get a difference */
 	dirdbGetName_internalstr (e1->file ? e1->file->dirdb_ref : e1->dir->dirdb_ref, &n1);
 	dirdbGetName_internalstr (e2->file ? e2->file->dirdb_ref : e2->dir->dirdb_ref, &n2);
 
+	return strcmp(n1, n2);
+}
+
+static int mlecmp_filesonly_groupdir (const void *a, const void *b)
+{
+	int _1 = *(int *)a;
+	int _2 = *(int *)b;
+	const struct modlistentry *e1 = &sorting->files[_1];
+	const struct modlistentry *e2 = &sorting->files[_2];
+
+	int i1 = mlecmp_score (e1);
+	int i2 = mlecmp_score (e2);
+
+	const char *n1, *n2;
+
+	int retval;
+
+	if (i1 != i2)
+	{
+		return i2 - i1;
+	}
+
+	if (e1->flags & MODLIST_FLAG_DRV)
+	{
+		return 0;
+	}
+	if (!e1->file->parent)
+	{
+		return 0;
+	}
+
+	retval = (int)((int32_t)e1->file->parent->dirdb_ref - (int32_t)e2->file->parent->dirdb_ref);
+	if (retval)
+	{
+		return retval;
+	}
+
+	dirdbGetName_internalstr (e1->file->dirdb_ref, &n1);
+	dirdbGetName_internalstr (e2->file->dirdb_ref, &n2);
+
 	return strcasecmp(n1, n2);
 }
 
@@ -454,6 +532,19 @@ void modlist_sort (struct modlist *modlist)
 	sorting = 0;
 }
 
+void modlist_subsort_filesonly_groupdir (struct modlist *modlist, unsigned int pos, unsigned int length)
+{
+	if ((pos >= modlist->num) ||
+	    (length > modlist->num) ||
+	    ((pos + length) > modlist->num))
+	{
+		return;
+	}
+	sorting = modlist; /* dirty HACK that is not thread-safe / reentrant what so ever */
+	qsort(modlist->sortindex + pos, length, sizeof(modlist->sortindex[0]), mlecmp_filesonly_groupdir);
+	sorting = 0;
+}
+
 struct modlist *modlist_create (void)
 {
 	/* TODO ARCS */
