@@ -1057,7 +1057,10 @@
 {0xA7C9, 0xA7CA},
 {0xA7CB, 0x0264},
 {0xA7CC, 0xA7CD},
+{0xA7CE, 0xA7CF},
 {0xA7D0, 0xA7D1},
+{0xA7D2, 0xA7D3},
+{0xA7D4, 0xA7D5},
 {0xA7D6, 0xA7D7},
 {0xA7D8, 0xA7D9},
 {0xA7DA, 0xA7DB},
@@ -1417,6 +1420,31 @@
 {0x16E5D, 0x16E7D},
 {0x16E5E, 0x16E7E},
 {0x16E5F, 0x16E7F},
+{0x16EA0, 0x16EBB},
+{0x16EA1, 0x16EBC},
+{0x16EA2, 0x16EBD},
+{0x16EA3, 0x16EBE},
+{0x16EA4, 0x16EBF},
+{0x16EA5, 0x16EC0},
+{0x16EA6, 0x16EC1},
+{0x16EA7, 0x16EC2},
+{0x16EA8, 0x16EC3},
+{0x16EA9, 0x16EC4},
+{0x16EAA, 0x16EC5},
+{0x16EAB, 0x16EC6},
+{0x16EAC, 0x16EC7},
+{0x16EAD, 0x16EC8},
+{0x16EAE, 0x16EC9},
+{0x16EAF, 0x16ECA},
+{0x16EB0, 0x16ECB},
+{0x16EB1, 0x16ECC},
+{0x16EB2, 0x16ECD},
+{0x16EB3, 0x16ECE},
+{0x16EB4, 0x16ECF},
+{0x16EB5, 0x16ED0},
+{0x16EB6, 0x16ED1},
+{0x16EB7, 0x16ED2},
+{0x16EB8, 0x16ED3},
 {0x1E900, 0x1E922},
 {0x1E901, 0x1E923},
 {0x1E902, 0x1E924},
