@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Routines for screen output
  *
@@ -36,6 +36,8 @@
  *     the generic functions is still kept in this file.
  */
 
+#define _POUTPUT_C
+
 #include "config.h"
 #include <stdio.h>
 #include <string.h>
@@ -51,7 +53,8 @@ unsigned char plpalette[256];
 const struct FontSizeInfo_t FontSizeInfo[] =
 {
 	{8, 8},
-	{8, 16}
+	{8, 16},
+	{16, 32}
 };
 
 void make_title (const char *part, int escapewarning)
@@ -59,7 +62,7 @@ void make_title (const char *part, int escapewarning)
 	char prebuf[32];
 	char buf[CONSOLE_MAX_X];
 	const char *title = "Open Cubic Player v" VERSION;
-	const char *copyright = "(c) 1994-'24 Stian Skjelstad";
+	const char *copyright = "(c) 1994-'26 Stian Skjelstad";
 
 	int spacem = plScrWidth - 2 - strlen (title) - strlen (copyright) - 2 - strlen (part);
 	int space1 = spacem / 2;
@@ -110,7 +113,7 @@ void generic_gdrawchar8p (uint16_t x, uint16_t y, uint8_t c, uint8_t f, void *pi
 
 	if (!picp)
 	{
-		gdrawchar8(x,y,c,f,0);
+		generic_gdrawchar8 (x,y,c,f,0);
 		return;
 	}
 
@@ -176,7 +179,7 @@ void generic_gdrawcharp (uint16_t x, uint16_t y, uint8_t c, uint8_t f, void *pic
 
 	if (!picp)
 	{
-		gdrawchar(x,y,c,f,0);
+		generic_gdrawchar (x,y,c,f,0);
 		return;
 	}
 
